@@ -7,7 +7,7 @@ from matplotlib import pyplot as plt
 from mrftools import DictionaryParameters, SequenceParameters
 
 class Simulation: 
-    def __init__(self,sequenceParameters, dictionaryParameters, name="", numTimepoints = -1, phaseRange=(-2*np.pi, 2*np.pi), numSpins=1, results=[], truncationMatrix=[], truncatedResults=[]):
+    def __init__(self,sequenceParameters, dictionaryParameters, name="", numTimepoints = -1, phaseRange=(-2*np.pi, 2*np.pi), numSpins=1, results=[], truncationMatrix=[], truncatedResults=[], singularValues=[]):
         self.sequenceParameters = sequenceParameters
         self.dictionaryParameters = dictionaryParameters
         if numTimepoints == -1:
@@ -18,6 +18,7 @@ class Simulation:
         self.results = results
         self.truncationMatrix = truncationMatrix
         self.truncatedResults = truncatedResults
+        self.singularValues = singularValues
         if not name:
             self.name = sequenceParameters.name + "_" + dictionaryParameters.name + "_" + str(numSpins) # Doesn't account for phase range
         else:
@@ -154,6 +155,18 @@ class Simulation:
         vt = np.transpose(v)
         self.truncationMatrix = vt[:,0:truncationNumber]
         self.truncatedResults = np.matmul(normalizedDictionary,self.truncationMatrix).transpose()
+        self.singularValues = s
+        return (u,s,v)
+
+    def GetTruncationNumberFromDesiredPower(self, desiredSVDPower):
+        singularVectorPowers = self.simulation.singularValues/np.sum(self.simulation.singularValues)
+        totalSVDPower=0; numSVDComponents=0
+        for singularVectorPower in singularVectorPowers:
+            totalSVDPower += singularVectorPower
+            numSVDComponents += 1
+            if totalSVDPower > desiredSVDPower:
+                break
+        return numSVDComponents, totalSVDPower
 
     def Export(self, filename, force=False, includeFullResults=True, includeSVDResults=True):
         if ".mrf" in filename:
@@ -210,6 +223,27 @@ class Simulation:
     def GetAverageResult(self, indices):
         return np.average(self.results[:,indices], 1)
 
+    def FindPatternMatches(self, querySignals, useSVD=False, truncationNumber=25):
+        if querySignals.ndim == 1:
+            querySignals = querySignals[:,None]
+        if not useSVD:
+            querySignalsTransposed = querySignals.transpose()
+            normalizedQuerySignal = querySignalsTransposed / np.linalg.norm(querySignalsTransposed, axis=1)[:,None]
+            simulationResultsTransposed = self.results.transpose()
+            normalizedSimulationResultsTransposed = simulationResultsTransposed / np.linalg.norm(simulationResultsTransposed, axis=1)[:,None]
+            innerProducts = np.inner(normalizedQuerySignal, normalizedSimulationResultsTransposed)
+            return np.argmax(abs(innerProducts), axis=1)
+        else:
+            if self.truncatedResults[:] == []:
+                self.CalculateSVD(truncationNumber)
+            signalsTransposed = querySignals.transpose()
+            signalSVDs = np.matmul(signalsTransposed, self.truncationMatrix)
+            normalizedQuerySignalSVDs = signalSVDs / np.linalg.norm(signalSVDs, axis=1)[:,None]
+            simulationResultsTransposed = self.truncatedResults.transpose()
+            normalizedSimulationResultsTransposed = simulationResultsTransposed / np.linalg.norm(simulationResultsTransposed, axis=1)[:,None]
+            innerProducts = np.inner(normalizedQuerySignalSVDs, normalizedSimulationResultsTransposed)
+            return np.argmax(abs(innerProducts), axis=1)
+
     @staticmethod
     def Import(filename, simulationName):
         if ".mrf" in filename:
